@@ -109,6 +109,8 @@ TickitTerm *tickit_term_new(void);
 TickitTerm *tickit_term_new_for_termtype(const char *termtype);
 void tickit_term_destroy(TickitTerm *tt);
 
+const char *tickit_term_get_termtype(TickitTerm *tt);
+
 void tickit_term_set_output_fd(TickitTerm *tt, int fd);
 int  tickit_term_get_output_fd(const TickitTerm *tt);
 void tickit_term_set_output_func(TickitTerm *tt, TickitTermOutputFunc *fn, void *user);
@@ -148,6 +150,14 @@ void tickit_term_setpen(TickitTerm *tt, const TickitPen *pen);
 void tickit_term_clear(TickitTerm *tt);
 void tickit_term_erasech(TickitTerm *tt, int count, int moveend);
 
+typedef enum {
+  TICKIT_TERMCTL_ALTSCREEN = 1,
+  TICKIT_TERMCTL_CURSORVIS,
+  TICKIT_TERMCTL_MOUSE,
+} TickitTermDriverCtl;
+
+int tickit_term_setctl_int(TickitTerm *tt, TickitTermDriverCtl ctl, int value);
+
 void tickit_term_set_mode_altscreen(TickitTerm *tt, int on);
 void tickit_term_set_mode_cursorvis(TickitTerm *tt, int on);
 void tickit_term_set_mode_mouse(TickitTerm *tt, int on);
